#!/usr/bin/python

import crypt
import Pyro.core
import time
import os
import sys
import shutil
from os import path
from decimal import Decimal, InvalidOperation

#import importerConf as opts
# Leo la config
if len(sys.argv) != 2:
    print >>sys.stderr, 'Usage: %s <configFile>\n' % sys.argv[0]
    sys.exit(1)


class cfg:
    pass


execfile(sys.argv[1], cfg.__dict__, cfg.__dict__)


class QueueServer(Pyro.core.ObjBase):

    def __init__(self, type='fifo', dir='.', validUsers={}, priorities=None):
        Pyro.core.ObjBase.__init__(self)
        self.type = type
        self.dir = dir
        if not path.isdir(self.dir):
            try:#crear el directorio de procesados si no existe
                os.mkdir(self.dir)
            except OSError:
                print 'No se pudo crear el directorio para almacenar los mensajes procesados'
                sys.exit(1)
        self.dirProcessed = path.join(self.dir, 'processed')
        if not path.isdir(self.dirProcessed):
            try:#crear el directorio de procesados si no existe
                os.mkdir(self.dirProcessed)
            except OSError:
                print 'No se pudo crear el directorio para almacenar los mensajes procesados'
                sys.exit(1)
        self.validUsers = validUsers
        self.priorities = priorities

    def _authenticate(self, loginInfo):
        '''
        Autenticar un usuario con su loginInfo (usuario, password)
        '''
        try:
            user = loginInfo[0]
            passwd = loginInfo[1]
        except IndexError:
            return False

        try:
            if self.validUsers[user] == crypt.crypt(passwd, passwd[:3]):
                return True
            else:
                return False
        except (IndexError, KeyError):
            return False

    def pop(self, loginInfo):
        '''
        Devolver un item del queue y borrarlo
        '''
        if not self._authenticate(loginInfo):
            raise RuntimeError("Invalid loginInfo")
        if self.priorities is None:
            # Cola sin prioridad
            i = self._nextItem()
            priority = None
        else:
            for prio in range(self.priorities):
                i = self._nextItem(prio)
                if not i is None:
                    priority = prio
                    break
        if i is None:
            return None

        if priority is None:
            dir = self.dir
            dirProcessed = self.dirProcessed
        else:
            dir = path.join(self.dir, "p" + str(priority))
            dirProcessed = path.join(self.dirProcessed, "p" + str(priority))
            if not path.isdir(dirProcessed):
                os.mkdir(dirProcessed)

        os.rename(path.join(dir, i[0]), path.join(dirProcessed, i[0]))
        return i[1]

    def peek(self, loginInfo):
        if not self._authenticate(loginInfo):
            raise RuntimeError("Invalid loginInfo")

        if self.priorities is None:
            # Cola sin prioridad
            i = self._nextItem()
            priority = None
        else:
            for prio in range(self.priorities):
                i = self._nextItem(prio)
                if not i is None:
                    priority = prio
                    break
        if i is None:
            return None

        i = self._nextItem()
        if not i == None:
            return i[1]
        else:
            return None

    def _nextItem(self, priority=None):
        '''
        Devolver un tuple (nombreDeArchivo, diccionario) del proximo item
        '''
        files = []
        if priority is None:
            dir = self.dir
        else:
            dir = path.join(self.dir, "p" + str(priority))
        try:
            dirList = os.listdir(dir)
        except OSError, e:
            return None # Dir not created
        for fileName in dirList:
            try: #no incluir los nombres de archivos que no son numericos
                Decimal(fileName)
                files.append(fileName)
            except (InvalidOperation, ValueError):
                continue

        files.sort(key=Decimal) #Organizar los nombres de archivos de menor a mayor

        if not files:
            return None
        if self.type == 'fifo':
            filename = files[0]
        else:
            filename = files[-1]

        f = file(os.path.join(dir, filename), 'r')
        txt = f.read()
        f.close()

        try: #Si no es codigo python valido, entonces devolverlo como string
            d = eval(txt)
        except SyntaxError:
            d = txt

        return (filename, d)

    def push(self, item, loginInfo, priority=None):
        '''
        Agregar un item al queue, recibe un diccionario, y lo guarda como
        string en un archivo cuyo nombre es el epoch en el que se crea
        '''
        if not self._authenticate(loginInfo):
            raise RuntimeError("Invalid loginInfo")
        if self.priorities is None:
            if not priority is None:
                raise RuntimeError("The queueServer was started without priorities.")
        else:
            if priority is None:
                priority = 0
            elif priority >= self.priorities:
                raise RuntimeError("Priority too high. Server has only %d priorities" % self.priorities)

        filename = str(time.time())
        if priority is None:
            dir = self.dir
        else:
            dir = path.join(self.dir, "p" + str(priority))
            if not path.isdir(dir):
                os.mkdir(dir)
        while path.isfile(path.join(dir, filename)): # Si el filename ya existe, agregar un
                                                     # 1 en la parte decimal
            filename = filename + "001"

        f = file(path.join(dir, filename), 'w')
        try:
            import pprint
            pprint.pprint(item, f)
        except:
            f.write(str(item))
        f.close()
        return True

if __name__ == '__main__':
    Pyro.core.initServer()
    q = Pyro.core.Daemon(port=cfg.pyroPort)
    uri = q.connect(QueueServer(type=cfg.type, dir=cfg.dir, validUsers=cfg.users,
                              priorities=getattr(cfg, "priorities", None)), "queueServer")
    q.requestLoop()
